package org.implab.gradle.containers.tasks;

import java.util.List;

import org.gradle.api.file.DirectoryProperty;
import org.gradle.api.file.RegularFileProperty;
import org.gradle.api.provider.Property;
import org.gradle.api.provider.Provider;
import org.gradle.api.tasks.Input;
import org.gradle.api.tasks.InputDirectory;
import org.gradle.api.tasks.Internal;
import org.gradle.api.tasks.OutputFile;
import org.gradle.api.tasks.SkipWhenEmpty;
import org.implab.gradle.Utils;
import org.implab.gradle.containers.ImageTag;

public abstract class BuildImage extends CliTask {
    @InputDirectory
    @SkipWhenEmpty
    public abstract DirectoryProperty getContextDirectory();

    @Internal
    public abstract Property<String> getImageQName();

    @Internal
    public abstract Property<String> getImageVersion();

    @Internal
    public abstract Property<String> getImageAuthority();

    @Input
    public Provider<ImageTag> getImageTag() {
        return getProject().provider(() -> {
            ImageTag tag = new ImageTag();
            tag.setQName(getImageQName().getOrNull());
            tag.setAuthority(getImageAuthority().getOrNull());
            tag.setVersion(getImageVersion().getOrNull());
            return tag;
        });
    }

    @OutputFile
    public abstract RegularFileProperty getImageIdFile();

    public BuildImage() {

        setWorkingDir(getContextDirectory().getAsFile());
    }

    @Override
    protected void preparingCommand(List<String> commandLine) {
        super.preparingCommand(commandLine);

        commandLine.addAll(List.of("build", ".", "-q", "-t"));
        commandLine.add(getImageTag().get().toString());
    }

    @Override
    protected void processStarted(Process p) {
        Utils.redirectIO(p.getInputStream(), getImageIdFile().get().getAsFile());
    }
}
